"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const events_1 = require("events");
const xmlParser = __importStar(require("fast-xml-parser"));
const http = __importStar(require("http"));
const node_ssdp_1 = require("node-ssdp");
const url = __importStar(require("url"));
const DEFAULT_TIMEOUT = 10000;
class RokuFinder extends events_1.EventEmitter {
    constructor() {
        super();
        this.intervalId = null;
        this.timeoutId = null;
        this.running = false;
        this.client = new node_ssdp_1.Client();
        this.client.on('response', (headers) => {
            if (!this.running) {
                return;
            }
            const { ST, LOCATION } = headers;
            if (ST && LOCATION && ST.indexOf('roku') !== -1) {
                http.get(`${LOCATION}/query/device-info`, {
                    headers: {
                        'User-Agent': 'https://github.com/adheus/roku-cli'
                    }
                }, (resp) => {
                    // Get the device info
                    let data = '';
                    resp.on('data', (chunk) => {
                        // A chunk of data has been received.
                        data += chunk;
                    });
                    resp.on('end', () => {
                        // The whole response has been received.
                        let info = xmlParser.parse(data);
                        const device = this.parseAddress(LOCATION);
                        device.deviceInfo = info['device-info'];
                        this.emit('found', device);
                    });
                });
            }
        });
    }
    start(timeout) {
        this.running = true;
        const search = () => {
            this.client.search('roku:ecp');
        };
        const done = () => {
            this.stop();
            this.emit('timeout');
        };
        search();
        this.intervalId = setInterval(search, 1000);
        this.timeoutId = setTimeout(done, timeout);
    }
    stop() {
        clearInterval(this.intervalId);
        clearTimeout(this.timeoutId);
        this.running = false;
        this.client.stop();
    }
    parseAddress(location) {
        const parts = url.parse(location);
        parts.path = undefined;
        parts.pathname = undefined;
        return { location: url.format(parts), ip: parts.hostname, deviceInfo: {} };
    }
}
//# sourceMappingURL=roku-discover.js.map