"use strict";
var __spreadArrays = (this && this.__spreadArrays) || function () {
    for (var s = 0, i = 0, il = arguments.length; i < il; i++) s += arguments[i].length;
    for (var r = Array(s), k = 0, i = 0; i < il; i++)
        for (var a = arguments[i], j = 0, jl = a.length; j < jl; j++, k++)
            r[k] = a[j];
    return r;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.LogLevel = exports.noop = exports.Logger = void 0;
var chalk_1 = __importDefault(require("chalk"));
var moment_1 = __importDefault(require("moment"));
var Stopwatch_1 = require("./Stopwatch");
var Logger = /** @class */ (function () {
    function Logger(logLevel) {
        /**
         * A string with whitespace used for indenting all messages
         */
        this.indent = '';
        this._logLevel = LogLevel.log;
        this.logLevel = logLevel;
    }
    Object.defineProperty(Logger.prototype, "logLevel", {
        get: function () {
            return this._logLevel;
        },
        set: function (value) {
            //cast the string version to the numberic version
            if (typeof (value) === 'string') {
                value = LogLevel[value];
            }
            this._logLevel = value !== null && value !== void 0 ? value : LogLevel.log;
        },
        enumerable: false,
        configurable: true
    });
    Logger.prototype.getTimestamp = function () {
        return '[' + chalk_1.default.grey(moment_1.default().format("hh:mm:ss:SSSS A")) + ']';
    };
    Logger.prototype.writeToLog = function (method) {
        var args = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            args[_i - 1] = arguments[_i];
        }
        if (this._logLevel === LogLevel.trace) {
            method = console.trace;
        }
        var finalArgs = [];
        for (var _a = 0, args_1 = args; _a < args_1.length; _a++) {
            var arg = args_1[_a];
            finalArgs.push(arg);
        }
        method.call.apply(method, __spreadArrays([console, this.getTimestamp(), this.indent], finalArgs));
    };
    /**
     * Log an error message to the console
     */
    Logger.prototype.error = function () {
        var messages = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            messages[_i] = arguments[_i];
        }
        if (this._logLevel >= LogLevel.error) {
            this.writeToLog.apply(this, __spreadArrays([console.error], messages));
        }
    };
    /**
     * Log a warning message to the console
     */
    Logger.prototype.warn = function () {
        var messages = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            messages[_i] = arguments[_i];
        }
        if (this._logLevel >= LogLevel.warn) {
            this.writeToLog.apply(this, __spreadArrays([console.warn], messages));
        }
    };
    /**
     * Log a standard log message to the console
     */
    Logger.prototype.log = function () {
        var messages = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            messages[_i] = arguments[_i];
        }
        if (this._logLevel >= LogLevel.log) {
            this.writeToLog.apply(this, __spreadArrays([console.log], messages));
        }
    };
    /**
     * Log an info message to the console
     */
    Logger.prototype.info = function () {
        var messages = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            messages[_i] = arguments[_i];
        }
        if (this._logLevel >= LogLevel.info) {
            this.writeToLog.apply(this, __spreadArrays([console.info], messages));
        }
    };
    /**
     * Log a debug message to the console
     */
    Logger.prototype.debug = function () {
        var messages = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            messages[_i] = arguments[_i];
        }
        if (this._logLevel >= LogLevel.debug) {
            this.writeToLog.apply(this, __spreadArrays([console.debug], messages));
        }
    };
    /**
     * Log a debug message to the console
     */
    Logger.prototype.trace = function () {
        var messages = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            messages[_i] = arguments[_i];
        }
        if (this._logLevel >= LogLevel.trace) {
            this.writeToLog.apply(this, __spreadArrays([console.trace], messages));
        }
    };
    /**
     * Writes to the log (if logLevel matches), and also times how long the action took to occur.
     * `action` is called regardless of logLevel, so this function can be used to nicely wrap
     * pieces of functionality.
     * The action function also includes two parameters, `pause` and `resume`, which can be used to improve timings by focusing only on
     * the actual logic of that action.
     */
    Logger.prototype.time = function (logLevel, messages, action) {
        var _this = this;
        //call the log if loglevel is in range
        if (this._logLevel >= logLevel) {
            var stopwatch_1 = new Stopwatch_1.Stopwatch();
            var logLevelString_1 = LogLevel[logLevel];
            //write the initial log
            this[logLevelString_1].apply(this, messages);
            this.indent += '  ';
            stopwatch_1.start();
            //execute the action
            var result_1 = action(stopwatch_1.stop.bind(stopwatch_1), stopwatch_1.start.bind(stopwatch_1));
            stopwatch_1.stop();
            //return a function to call when the timer is complete
            var done = function () {
                _this.indent = _this.indent.substring(2);
                _this[logLevelString_1].apply(_this, __spreadArrays(messages !== null && messages !== void 0 ? messages : [], ["finished. (" + chalk_1.default.blue(stopwatch_1.getDurationText()) + ")"]));
            };
            //if this is a promise, wait for it to resolve and then return the original result
            if (typeof (result_1 === null || result_1 === void 0 ? void 0 : result_1.then) === 'function') {
                return Promise.resolve(result_1).then(done).then(function () {
                    return result_1;
                });
            }
            else {
                //this was not a promise. finish the timer now
                done();
                return result_1;
            }
        }
        else {
            return action(noop, noop);
        }
    };
    return Logger;
}());
exports.Logger = Logger;
function noop() {
}
exports.noop = noop;
var LogLevel;
(function (LogLevel) {
    LogLevel[LogLevel["off"] = 0] = "off";
    LogLevel[LogLevel["error"] = 1] = "error";
    LogLevel[LogLevel["warn"] = 2] = "warn";
    LogLevel[LogLevel["log"] = 3] = "log";
    LogLevel[LogLevel["info"] = 4] = "info";
    LogLevel[LogLevel["debug"] = 5] = "debug";
    LogLevel[LogLevel["trace"] = 6] = "trace";
})(LogLevel = exports.LogLevel || (exports.LogLevel = {}));
//# sourceMappingURL=Logger.js.map