"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Stopwatch = void 0;
var parse_ms_1 = __importDefault(require("parse-ms"));
var perf_hooks_1 = require("perf_hooks");
var Stopwatch = /** @class */ (function () {
    function Stopwatch() {
        this.totalMilliseconds = 0;
    }
    Stopwatch.prototype.start = function () {
        this.startTime = perf_hooks_1.performance.now();
    };
    Stopwatch.prototype.stop = function () {
        if (this.startTime) {
            this.totalMilliseconds += perf_hooks_1.performance.now() - this.startTime;
        }
        this.startTime = undefined;
    };
    Stopwatch.prototype.reset = function () {
        this.totalMilliseconds = 0;
        this.startTime = undefined;
    };
    Stopwatch.prototype.getDurationText = function () {
        var parts = parse_ms_1.default(this.totalMilliseconds);
        var fractionalMilliseconds = parseInt(this.totalMilliseconds.toFixed(3).toString().split('.')[1]);
        if (parts.minutes > 0) {
            return parts.minutes + "m" + parts.seconds + "s" + parts.milliseconds + "." + fractionalMilliseconds + "ms";
        }
        else if (parts.seconds > 0) {
            return parts.seconds + "s" + parts.milliseconds + "." + fractionalMilliseconds + "ms";
        }
        else {
            return parts.milliseconds + "." + fractionalMilliseconds + "ms";
        }
    };
    return Stopwatch;
}());
exports.Stopwatch = Stopwatch;
//# sourceMappingURL=Stopwatch.js.map